<?php

namespace Drupal\webform_mailerlite\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Site\Settings;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use MailerLite\MailerLite;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form submission to MailerLite handler.
 *
 * @WebformHandler(
 *   id = "mailerlite",
 *   label = @Translation("MailerLite"),
 *   category = @Translation("MailerLite"),
 *   description = @Translation("Sends a form submission to a MailerLite list."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class WebformMailerliteHandler extends WebformHandlerBase {

  /**
   * The Mailerlite client.
   *
   * @var \MailerLite\MailerLite
   */
  protected MailerLite $mailerlite;

  /**
   * The token manager.
   *
   * @var \Drupal\webform\WebformTokenManagerInterface
   */
  protected $tokenManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->tokenManager = $container->get('webform.token_manager');
    $key = Settings::get("mailerlite_key", "");
    $instance->mailerlite = new MailerLite(['api_key' => $key]);

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $key = Settings::get("mailerlite_key", "");
    $fields = $this->getWebform()->getElementsInitializedAndFlattened();

    $key = substr($key, 0, 10);
    $key_summary = '<strong>' . $this->t('Key') . ': </strong>' . $key;

    $email_summary = $this->configuration['email'];
    if (!empty($fields[$this->configuration['email']])) {
      $email_summary = $fields[$this->configuration['email']]['#title'];
    }
    $email_summary = '<strong>' . $this->t('Email') . ': </strong>' . $email_summary;

    $markup = "$key_summary<hr>$email_summary";
    return [
      '#markup' => $markup,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'list' => '',
      'email' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $enabled = $this->configFactory->get('mailerlite.enabled_lists')->getRawData();

    $form['mailerlite'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('MailerLite settings'),
      '#attributes' => ['id' => 'webform-mailerlite-handler-settings'],
    ];

    $fields = $this->getWebform()->getElementsInitializedAndFlattened();
    $options = [];
    foreach ($fields as $field_name => $field) {
      if (in_array($field['#type'], ['email', 'webform_email_confirm'])) {
        $options[$field_name] = $field['#title'];
      }
    }

    $default_value = $this->configuration['email'];
    if (empty($this->configuration['email']) && count($options) == 1) {
      $default_value = key($options);
    }
    $form['mailerlite']['email'] = [
      '#type' => 'webform_select_other',
      '#title' => $this->t('Email field'),
      '#required' => TRUE,
      '#default_value' => $default_value,
      '#options' => $options,
      '#empty_option'=> $this->t('- Select an option -'),
      '#description' => $this->t('Select the email element you want to use for subscribing to the MailerLite list specified above.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValues();
    foreach ($this->configuration as $name => $value) {
      if (isset($values['mailerlite'][$name])) {
        $this->configuration[$name] = $values['mailerlite'][$name];
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    if ($update) {
      return;
    }

    $fields = $webform_submission->toArray(TRUE);
    $configuration = $this->tokenManager->replace($this->configuration, $webform_submission);

    // Email could be a webform element or a string/token.
    if (!empty($fields['data'][$configuration['email']])) {
      $email = $fields['data'][$configuration['email']];
    }
    else {
      $email = $configuration['email'];
    }

    if (!empty($email)) {
      $data = array_merge(['email' => $email]);
      $this->mailerlite->subscribers->create($data);
    } else {
      \Drupal::logger('webform_submission')->warning(
        'webform_mailerlite: No email address was provided to the handler.',
      );
    }
  }

}
